from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *


mdb.ModelFromInputFile(inputFileName=
    'path/CM4.inp', name=
    'CM4')
Mod = mdb.models['CM4']

del mdb.models['Model-1']

Mod.interactions.delete((
    'CT_MENLATT_WITH_FCARTT-1', 'CT_MENLATT_WITH_TCARTLATT-1', 
    'CT_MENMEDT_WITH_FCARTT-1', 'CT_MENMEDT_WITH_TCARTMEDT-1'))
del Mod.interactionProperties['CT_MENLATT_WITH_FCARTT']
del Mod.interactionProperties['CT_MENLATT_WITH_TCARTLATT']
del Mod.interactionProperties['CT_MENMEDT_WITH_FCARTT']
del Mod.interactionProperties['CT_MENMEDT_WITH_TCARTMEDT']

# interactions
Mod.ContactProperty('IntProp-1')
Mod.interactionProperties['IntProp-1'].NormalBehavior(
	allowSeparation=ON, constraintEnforcementMethod=DEFAULT, 
	pressureOverclosure=HARD)
Mod.interactionProperties['IntProp-1'].TangentialBehavior(
	dependencies=0, directionality=ISOTROPIC, elasticSlipStiffness=None, 
	formulation=PENALTY, fraction=0.005, maximumElasticSlip=FRACTION, 
	pressureDependency=OFF, shearStressLimit=None, slipRateDependency=OFF, 
	table=((0.1, ), ), temperatureDependency=OFF)		
	
Mod.ContactProperty('IntProp-2')
Mod.interactionProperties['IntProp-2'].NormalBehavior(
	allowSeparation=ON, constraintEnforcementMethod=DEFAULT, 
	pressureOverclosure=HARD)
Mod.interactionProperties['IntProp-2'].TangentialBehavior(
	dependencies=0, directionality=ISOTROPIC, elasticSlipStiffness=None, 
	formulation=PENALTY, fraction=0.005, maximumElasticSlip=FRACTION, 
	pressureDependency=OFF, shearStressLimit=None, slipRateDependency=OFF, 
	table=((0.15, ), ), temperatureDependency=OFF)			
	
# surface interactions
Mod.SurfaceToSurfaceContactStd(adjustMethod=OVERCLOSED, 
	clearanceRegion=None, createStepName='Initial', datumAxis=None, 
	initialClearance=OMIT, interactionProperty='IntProp-2', master=
	Mod.rootAssembly.surfaces['SF_FCARTT_SELF_CONTACT'], name='Int-1', 
	slave=Mod.rootAssembly.surfaces['SF_MENLATT_SELF_CONTACT'], 
	sliding=FINITE, thickness=ON)
	
Mod.SurfaceToSurfaceContactStd(adjustMethod=OVERCLOSED, 
	clearanceRegion=None, createStepName='Initial', datumAxis=None, 
	initialClearance=OMIT, interactionProperty='IntProp-2', master=
	Mod.rootAssembly.surfaces['SF_FCARTT_SELF_CONTACT'], name='Int-2', 
	slave=Mod.rootAssembly.surfaces['SF_MENMEDT_SELF_CONTACT'], 
	sliding=FINITE, thickness=ON)		
	
Mod.SurfaceToSurfaceContactStd(adjustMethod=OVERCLOSED, 
	clearanceRegion=None, createStepName='Initial', datumAxis=None, 
	initialClearance=OMIT, interactionProperty='IntProp-2', master=
	Mod.rootAssembly.surfaces['SF_TCARTLATT_SELF_CONTACT'], name='Int-3', 
	slave=Mod.rootAssembly.surfaces['SF_MENLATT_SELF_CONTACT'], 
	sliding=FINITE, thickness=ON)		

Mod.SurfaceToSurfaceContactStd(adjustMethod=OVERCLOSED, 
	clearanceRegion=None, createStepName='Initial', datumAxis=None, 
	initialClearance=OMIT, interactionProperty='IntProp-2', master=
	Mod.rootAssembly.surfaces['SF_TCARTMEDT_SELF_CONTACT'], name='Int-4', 
	slave=Mod.rootAssembly.surfaces['SF_MENMEDT_SELF_CONTACT'], 
	sliding=FINITE, thickness=ON)	
	
Mod.SurfaceToSurfaceContactStd(adjustMethod=OVERCLOSED, 
	clearanceRegion=None, createStepName='Initial', datumAxis=None, 
	initialClearance=OMIT, interactionProperty='IntProp-1', master=
	Mod.rootAssembly.surfaces['SF_FCARTT_SELF_CONTACT'], name='Int-5', 
	slave=Mod.rootAssembly.surfaces['SF_TCARTMEDT_SELF_CONTACT'], 
	sliding=FINITE, thickness=ON)	

Mod.SurfaceToSurfaceContactStd(adjustMethod=OVERCLOSED, 
	clearanceRegion=None, createStepName='Initial', datumAxis=None, 
	initialClearance=OMIT, interactionProperty='IntProp-1', master=
	Mod.rootAssembly.surfaces['SF_FCARTT_SELF_CONTACT'], name='Int-6', 
	slave=Mod.rootAssembly.surfaces['SF_TCARTLATT_SELF_CONTACT'], 
	sliding=FINITE, thickness=ON)	

# reference points	
RPfemur = Mod.rootAssembly.ReferencePoint(point=(55,41.4,65)).id
RPtibia = Mod.rootAssembly.ReferencePoint(point=(47.03043,51.741821,-5)).id		

# create steps after initial
Mod.StaticStep(name='Move', previous='Initial', matrixSolver=DIRECT, matrixStorage=UNSYMMETRIC)
Mod.StaticStep(name='Load', previous='Move', matrixSolver=DIRECT, matrixStorage=UNSYMMETRIC)
Mod.steps['Move'].setValues(nlgeom=ON)	

# materials
Mod.materials['PM_FCARTT'].Hyperelastic(materialType=ISOTROPIC, 
    table=((0.6849, 0.12), ), testData=OFF, type=NEO_HOOKE, volumetricResponse=
    VOLUMETRIC_DATA)
Mod.materials['PM_TCARTLATT'].Hyperelastic(materialType=ISOTROPIC, 
    table=((0.6849, 0.12), ), testData=OFF, type=NEO_HOOKE, volumetricResponse=
    VOLUMETRIC_DATA)
Mod.materials['PM_TCARTMEDT'].Hyperelastic(materialType=ISOTROPIC, 
    table=((0.6849, 0.12), ), testData=OFF, type=NEO_HOOKE, volumetricResponse=
    VOLUMETRIC_DATA)

Mod.materials['PM_MENLATT'].Elastic(table=((4.0, 4.0, 14, 0.3, 0.2, 0.2, 5.3846, 1.66, 1.66), ), type=ENGINEERING_CONSTANTS)
Mod.materials['PM_MENMEDT'].Elastic(table=((4.0, 4.0, 14, 0.3, 0.2, 0.2, 5.3846, 1.66, 1.66), ), type=ENGINEERING_CONSTANTS)
# orientation must be added manually

Mod.materials['PM_FEMURT'].Elastic(table=((15000.0, 0.4), ))
Mod.materials['PM_TIBIAFT'].Elastic(table=((15000.0, 0.4), ))

# mesh
mdb.models['CM4'].parts['PART-1'].setElementType(elemTypes=(ElemType(
    elemCode=C3D10, elemLibrary=STANDARD), ), regions=(
    mdb.models['CM4'].parts['PART-1'].elements.getByBoundingSphere( (0,0,0), 500), ))
	
# couplings
mdb.models['CM4'].Coupling(controlPoint=Region(referencePoints=(
    mdb.models['CM4'].rootAssembly.referencePoints[RPfemur], )), couplingType=
    KINEMATIC, influenceRadius=WHOLE_SURFACE, localCsys=None, name='Constraint-1', surface=
    mdb.models['CM4'].rootAssembly.sets['NS_FEMURT_WITH_ZMAX'], u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)	
	
mdb.models['CM4'].Coupling(controlPoint=Region(referencePoints=(
    mdb.models['CM4'].rootAssembly.referencePoints[RPtibia], )), couplingType=
    KINEMATIC, influenceRadius=WHOLE_SURFACE, localCsys=None, name='Constraint-2', surface=
    mdb.models['CM4'].rootAssembly.sets['NS_TIBIAFT_WITH_ZMIN'], u1=ON, u2=ON, u3=ON, ur1=ON, ur2=ON, ur3=ON)		

# BCs

# for flexion rotation, edit ur1 to > 0	
Mod.DisplacementBC(amplitude=UNSET, createStepName='Move', 
    distributionType=UNIFORM, fieldName='', fixed=OFF, localCsys=None, name=
    'Displacement', region=Region(referencePoints=(Mod.rootAssembly.referencePoints[RPfemur], )), u1=0.0, u2=
    0.0, u3=-0.1, ur1=0.0, ur2=0.0, ur3=0.0)

Mod.ConcentratedForce(cf3=-500.0, createStepName='Load', 
    distributionType=UNIFORM, field='', localCsys=None, name='Load-1', region=
    Region(referencePoints=(Mod.rootAssembly.referencePoints[RPfemur], )))

Mod.DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', fixed=OFF, localCsys=None, name=
    'Fix_tib', region=Region(referencePoints=(Mod.rootAssembly.referencePoints[RPtibia], )), u1=0.0, u2=
    0.0, u3=0.0, ur1=0.0, ur2=0.0, ur3=0.0)

# activate this for load step if editing Fix-tib in move step to allow freedom	
Mod.VelocityBC(amplitude=UNSET, createStepName='Load', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='refix-tib', 
    region=Region(referencePoints=(
    mdb.models['CM4'].rootAssembly.referencePoints[RPtibia], )), v1=0.0, v2=0.0, 
    v3=0.0, vr1=0.0, vr2=0.0, vr3=0.0)
Mod.boundaryConditions['refix-tib'].suppress()	

# only use for 0 flexion cases, otherwise just unset u3 for load
Mod.boundaryConditions['Displacement'].deactivate('Load')

# copy to generate meniscectomy model
mdb.Model(name='NoMen', objectToCopy=mdb.models['CM4'])
mdb.meshEditOptions.setValues(enableUndo=True, maxUndoCacheElements=0.5)
mdb.models['NoMen'].parts['PART-1'].deleteElement(elements=
    [mdb.models['NoMen'].parts['PART-1'].sets['PT_MENLATT'], 
    mdb.models['NoMen'].parts['PART-1'].sets['PT_MENMEDT']])
mdb.models['NoMen'].interactions.delete(('Int-1', 'Int-2', 'Int-3', 'Int-4'))

# outputs
mdb.models['CM4'].historyOutputRequests['H-Output-1'].setValuesInStep(stepName=
    'Load', variables=('CAREA', 'ALLAE', 'ALLCD', 'ALLDMD', 'ALLEE', 'ALLFD', 
    'ALLIE', 'ALLJD', 'ALLKE', 'ALLKL', 'ALLPD', 'ALLQB', 'ALLSE', 'ALLSD', 
    'ALLVD', 'ALLWK', 'ETOTAL'))
mdb.models['CM4'].fieldOutputRequests['F-Output-1'].setValuesInStep(stepName=
    'Load', variables=('S', 'PE', 'PEEQ', 'PEMAG', 'LE', 'U', 'RF', 'CF', 
    'CSTRESS', 'CDISP', 'CFORCE', 'CNAREA'))

# job
Mod.rootAssembly.regenerate()		
myJob = mdb.Job(name='cm4', model='CM4')
myJob.setValues(memory=90, memoryUnits=PERCENTAGE)
myJob.setValues(numCpus=16,numDomains=16,multiprocessingMode=THREADS)
